"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.ChangeType = void 0;

var _parse = _interopRequireDefault(require("parse5"));

var _blockContentToHtml = _interopRequireDefault(require("@sanity/block-content-to-html"));

var _imageUrl = _interopRequireDefault(require("@sanity/image-url"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var BlockTypeCollection = 'collection';

class ChangeType {}

exports.ChangeType = ChangeType;

_defineProperty(ChangeType, "UPDATE_OR_CREATE", 'update_or_create');

_defineProperty(ChangeType, "DELETE", 'delete');

class SanityFetcher {
  /**
   * @param sanityConfig {object} config info for sanity
   * @param blocks {Map<string, Block>} mapping between type to block schema
   */
  constructor(sanityClient, blocks) {
    this.sanity = sanityClient;
    this.sanityConfig = sanityClient.config();
    this.blocks = getBlocksMap(blocks);
    this.query = '*[_type in $types] { ..., "order": coalesce(order, 0) } | order(order asc)';
    this.params = {
      types: Object.keys(this.blocks)
    };
    console.log('[fetcher] new instance', this.query, this.params, this.blocks.length);
  }
  /*
   * @returns {Promise<object>} a map from blockName -> document | list of documents 
   */


  fetchRecords() {
    var _this = this;

    return _asyncToGenerator(function* () {
      console.log('[fetcher] this?', _this); // sanity returns a flat list of documents
      // the _type field is equivilent to the name of the field in interneto block

      var sanityDocs = yield _this.sanity.fetch(_this.query, _this.params);
      console.log('[fetcher] sanity docs:', sanityDocs);
      var records = {};

      var _iterator = _createForOfIteratorHelper(sanityDocs),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var sanityDoc = _step.value;
          _this.blocks[sanityDoc._type];

          if (_this.blocks[sanityDoc._type].type == BlockTypeCollection) {
            records[sanityDoc._type] = records[sanityDoc._type] || [];

            records[sanityDoc._type].push(yield _this._prepareDoc(sanityDoc));

            Object.assign(records, yield _this._prepareDirectChildren(sanityDoc));
            continue;
          }

          records[sanityDoc._type] = yield _this._prepareDoc(sanityDoc);
          Object.assign(records, yield _this._prepareDirectChildren(sanityDoc));
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return records;
    })();
  }

  enrichSanityDoc(records, sanityDoc) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      var docClone = Object.assign({}, sanityDoc);
      var enrichedDoc = yield _this2._prepareDoc(docClone);
      records[sanityDoc._type] = enrichedDoc;
      var directChildren = yield _this2._prepareDirectChildren(sanityDoc);
      console.log('direct children', directChildren);
      Object.assign(records[sanityDoc._type], directChildren);
      return records;
    })();
  }

  _prepareDirectChildren(doc) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      var block = _this3.blocks[doc._type];

      if (block === undefined) {
        return {};
      }

      return yield _this3._prepareDirectChildrenWithBlock(doc, block);
    })();
  }

  _prepareDirectChildrenWithBlock(doc, block) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      if (!block.directChildren) {
        return {};
      }

      var internalRecords = {};

      var _iterator2 = _createForOfIteratorHelper(block.directChildren),
          _step2;

      try {
        var _loop = function* _loop() {
          var childBlock = _step2.value;

          if (!(childBlock.id in doc)) {
            return "continue";
          }

          if (childBlock.type === BlockTypeCollection) {
            var childDocs = doc[childBlock.id];
            internalRecords[childBlock.id] = yield Promise.all(childDocs.map(doc => {
              if (doc.directChildren !== undefined && doc.directChildren.length !== 0) {
                throw new Error("Unimlemented, direct children of child array");
              }

              return _this4._prepareDocWithBlock(doc, childBlock);
            }));
            return "continue";
          }

          var childDoc = doc[childBlock.id];
          internalRecords[childBlock.id] = yield _this4._prepareDocWithBlock(childDoc, childBlock);
          Object.assign(internalRecords, yield _this4._prepareDirectChildrenWithBlock(childDoc, childBlock));
        };

        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var _ret = yield* _loop();

          if (_ret === "continue") continue;
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }

      return internalRecords;
    })();
  }

  _prepareDoc(doc) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      var block = _this5.blocks[doc._type];
      return yield _this5._prepareDocWithBlock(doc, block);
    })();
  }

  _prepareDocWithBlock(originalDoc, block) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      var doc = Object.assign({}, originalDoc);

      if (block === undefined) {
        throw new Error("no block with type ".concat(doc._type));
      }

      var _iterator3 = _createForOfIteratorHelper(block.fields),
          _step3;

      try {
        for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
          var field = _step3.value;

          if (!(field.id in doc)) {
            // field is in schema decleration but not in document
            continue;
          } // copy value so if we edit it we dont change the original field type


          var fieldType = field.type; // handle references before other types so we can handle references to images too

          if (fieldType === 'reference' && Array.isArray(doc[field.id]) === true) {
            var refs = doc[field.id].map(d => d._ref);
            var refDocs = yield _this6.sanity.fetch('*[_id in $refs]', {
              refs: refs
            });
            doc[field.id] = refDocs;
            fieldType = 'array';
          } else if (fieldType === 'reference') {
            var refDoc = yield _this6.sanity.fetch('*[_id == $_ref]', {
              _ref: doc[field.id]._ref
            });
            doc[field.id] = refDoc;
          }

          if (fieldType === 'array') {
            var newDocs = []; // recursive resolving for array of references

            var _iterator4 = _createForOfIteratorHelper(doc[field.id]),
                _step4;

            try {
              for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
                var innerDoc = _step4.value;
                newDocs.push(yield _this6._prepareDoc(innerDoc));
              }
            } catch (err) {
              _iterator4.e(err);
            } finally {
              _iterator4.f();
            }

            doc[field.id] = newDocs;
          }

          if (fieldType === 'html') {
            doc[field.id] = _this6._prepareHtmlField(doc[field.id]);
          }

          if (fieldType === 'image') {
            doc[field.id] = _this6._prepareImageField(doc[field.id]);
          }

          if (fieldType === 'video') {
            doc[field.id] = yield _this6._prepareVideoField(doc[field.id]);
          }
        }
      } catch (err) {
        _iterator3.e(err);
      } finally {
        _iterator3.f();
      }

      if (block.type == BlockTypeCollection && doc.slug) {
        doc._permalink = "/".concat(doc._type, "/").concat(doc.slug.current);
      }

      return doc;
    })();
  }

  _prepareHtmlField(source) {
    return (0, _blockContentToHtml.default)({
      blocks: source,
      projectId: this.sanityConfig.projectId,
      dataset: this.sanityConfig.dataset,
      serializers: {
        types: {
          code: this._prepareHtmlCodeSection.bind(this),
          raw: this._prepareRawSection.bind(this),
          table: this._prepareTableSection.bind(this)
        }
      }
    });
  }

  _prepareTableSection(props) {
    var h = _blockContentToHtml.default.h;
    var rows = props.node.rows;

    if (rows.length < 1) {
      return '';
    }

    return h('table', h('tr', rows[0].cells.map(cell => h('th', cell))), rows.slice(1).map(row => {
      return h('tr', row.cells.map(cell => h('td', cell)));
    }));
  }

  _prepareHtmlCodeSection(props) {
    // `h` is a way to build HTML known as hyperscript
    var h = _blockContentToHtml.default.h;
    var className = "".concat(props.node.language, " language-").concat(props.node.language);
    return h('pre', {
      className: className
    }, h('code', {
      className: className
    }, props.node.code));
  }

  _prepareRawSection(props) {
    return htmlToHyperscript(props.node.code);
  }

  _prepareImageField(source) {
    var builder = (0, _imageUrl.default)(this.sanity);
    return builder.image(source).url();
  }

  _prepareVideoField(source) {
    var _this7 = this;

    return _asyncToGenerator(function* () {
      var assetDocument = yield _this7.sanity.getDocument(source.asset._ref);
      return "mux:".concat(assetDocument.playbackId);
    })();
  }

}

exports.default = SanityFetcher;

function htmlToHyperscript(html) {
  var h = _blockContentToHtml.default.h;

  function parse5ToHyperscript(doc) {
    if (doc === undefined) {
      return undefined;
    } // return the actual content of the node, has no tag name as it is just text


    if (!doc.tagName && typeof doc.value === 'string') {
      return doc.value;
    }

    var children = [];

    if (Array.isArray(doc.childNodes)) {
      children = doc.childNodes.map(node => parse5ToHyperscript(node));
    }

    var attrs = {};
    doc.attrs.map(attr => attrs[attr.name] = attr.value);
    return h(doc.tagName, attrs, children);
  }

  var document = _parse.default.parseFragment("<div class=\"embed\">".concat(html, "</div>"));

  var documentHtml = document.childNodes[0];
  var out = parse5ToHyperscript(documentHtml);
  return out;
} // return mapping block.id -> block


var getBlocksMap = originalBlocks => {
  var blocks = {};

  var _iterator5 = _createForOfIteratorHelper(originalBlocks),
      _step5;

  try {
    for (_iterator5.s(); !(_step5 = _iterator5.n()).done;) {
      var block = _step5.value;
      blocks[block.id] = block;
    }
  } catch (err) {
    _iterator5.e(err);
  } finally {
    _iterator5.f();
  }

  return blocks;
};
//# sourceMappingURL=fetcher.js.map